package net.wanji.opt.servicev2.report.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.dao.mapper.report.GreenWaveWeekDataMapper;
import net.wanji.opt.entity.report.ConfigPeakHours;
import net.wanji.opt.entity.report.GreenWaveWeekData;
import net.wanji.opt.servicev2.report.GreenWaveWeekDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.WeekFields;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

@Service
@Slf4j
public class GreenWaveWeekDataServiceImpl extends ServiceImpl<GreenWaveWeekDataMapper, GreenWaveWeekData>
        implements GreenWaveWeekDataService {
    @Autowired
    private GreenWaveWeekDataService greenWaveWeekDataService;
    @Autowired
    private GreenWaveWeekDataMapper greenWaveWeekDataMapper;

    @Override
    public void insertGreenWaveWeekData() {
        List<ConfigPeakHours> configPeakHours = greenWaveWeekDataMapper.selectConfigPeakHoursAll();
        // 获取上周的开始时间（周一 00:00:00）
        LocalDateTime startOfLastWeek = getStartOfLastWeek();
        // 获取上周的结束时间（周日 23:59:59）
        LocalDateTime endOfLastWeek = getEndOfLastWeek();

        // 格式化输出
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        String startDate = startOfLastWeek.format(formatter);
        String endDate = endOfLastWeek.format(formatter);
        for (ConfigPeakHours configPeakHour : configPeakHours) {
            String startTime = configPeakHour.getStartTime();
            String endTime = configPeakHour.getEndTime();
            List<GreenWaveWeekData> greenWaveWeekDataList = greenWaveWeekDataMapper.selectGreenWaveWeekDataByTime(startDate, endDate,startTime,endTime);

            greenWaveWeekDataList.forEach(x -> {
                x.setPeakType(String.valueOf(configPeakHour.getPeakType()));
                x.setDayType(String.valueOf(configPeakHour.getDayType()));
                x.setStartTime(startDate);
                x.setEndTime(endDate);
                Integer dir = getDir(x.getRoadDirection());
                Integer flow = greenWaveWeekDataMapper.getFlowByTime(x.getGreenId(), dir, startDate, endDate,startTime,endTime);
                x.setFlow(flow);
                x.setYearWeek(getWeekNumberOfYear(startOfLastWeek));
                x.setInsertTime(new Date());
            });
            greenWaveWeekDataService.insertOrUpdateBatch(greenWaveWeekDataList);
            log.info("周总体拥堵高峰期概况批量保存或更新成功！");
        }
        List<GreenWaveWeekData> greenWaveWeekDataList = greenWaveWeekDataMapper.selectGreenWaveWeekData(startDate, endDate);
        greenWaveWeekDataList.forEach(x -> {
            x.setPeakType("1");
            x.setDayType("3");
            x.setStartTime(startDate);
            x.setEndTime(endDate);
            Integer dir = getDir(x.getRoadDirection());
            Integer flow = greenWaveWeekDataMapper.getFlow(x.getGreenId(), dir, startDate, endDate);
            x.setFlow(flow);
            x.setYearWeek(getWeekNumberOfYear(startOfLastWeek));
            x.setInsertTime(new Date());
        });
        greenWaveWeekDataService.insertOrUpdateBatch(greenWaveWeekDataList);
        log.info("周总体拥堵高峰期概况批量保存或更新成功！");
    }

    @Override
    public void insertOrUpdateBatch(List<GreenWaveWeekData> greenWaveWeekDataList) {
        greenWaveWeekDataMapper.insertOrUpdateBatch(greenWaveWeekDataList);
    }


    private Integer getDir(String roadDirection) {
        switch (roadDirection) {
            case "e2w":
                return 3;
            case "w2e":
                return 7;
            case "s2n":
                return 5;
            case "n2s":
                return 1;
            default:
                return 3;
        }
    }

    /*
     * 获取周数
     * */
    public static int getWeekNumberOfYear(LocalDateTime date) {
        WeekFields weekFields = WeekFields.of(Locale.getDefault()); // 使用默认区域设置
        return date.get(weekFields.weekOfWeekBasedYear());
    }

    /*
     * 获取日期yyyy-MM-dd
     * */
    public List<String> getLastWeeKDays() {
        LocalDate today = LocalDate.now();
        LocalDate lastMonday = today.with(DayOfWeek.MONDAY).minusWeeks(1);

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

        List<String> dayList = new ArrayList<>();
        for (int i = 0; i < 7; i++) {
            LocalDate day = lastMonday.plusDays(i);
            dayList.add(day.format(formatter));
        }
        return dayList;
    }

    /**
     * 获取上周的开始时间（周一 00:00:00）
     */
    public static LocalDateTime getStartOfLastWeek() {
        // 获取当前日期
        LocalDateTime now = LocalDateTime.now();
        // 找到上周一
        LocalDateTime lastMonday = now.with(DayOfWeek.MONDAY).minusWeeks(1)
                .with(LocalTime.MIN); // 设置时间为 00:00:00
        return lastMonday;
    }

    /**
     * 获取上周的结束时间（周日 23:59:59）
     */
    public static LocalDateTime getEndOfLastWeek() {
        // 获取当前日期
        LocalDateTime now = LocalDateTime.now();
        // 找到上周日
        LocalDateTime lastSunday = now.with(DayOfWeek.SUNDAY).minusWeeks(1)
                .with(LocalTime.MAX); // 设置时间为 23:59:59
        return lastSunday;
    }
}